#!/usr/bin/perl -w

# review.cgi: review, evaluate and annotate applications of Faculty applicants
#             also allows to (manually!) send email to references to request letters
#
# Copyright (C) 2000-2003 Filippo Menczer and the University of Iowa
# Copyright (C) 2003-2004 Filippo Menczer and Indiana University
#
#    This file is part of the Recruit system
#
#    Recruit is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Recruit is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Recruit; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# CONVENTIONS: 
# - admin password (encrypted) is stored in file 'passwd'
# - applicant's info is stored in database $ID/$ID.DB
# - applicant's CV is stored in file $path/$ID/$ID-cvfile.pfd
# - applicant's statement is stored in file $path/$ID/$ID-stmtfile.pfd
# - applicant's recommendation letter X is stored in file $path/$ID/$ID-letter-X.(txt|pdf)
# - committee info is stored in database 'committee.DB'
# - evaluations (intersection table) are stored in 'eval.DB'

$|=1;
use strict;
use CGI::Carp qw(fatalsToBrowser);
use CGI qw(:standard);
use DB_File;

# PARAMETERS OF THE SEARCH
use lib qw(.);
use params ();
use vars qw(%global);
*global = \%params::global;
my $dept = $global{dept};
my $path = $global{path};
my $URLpath = $global{URLpath};
my $arealist = $global{areas};
my $passwdflag = $global{passwdflag};

# return javascript to pop up comments window
sub js_showcomment {
my $JSCRIPT=<<END;
// Input c: candidate (use for title)
// Input s: comment string (should have <p> separators between reviews)
function showcomment(c,s) {
   cm = window.open("", c, "toolbar=no,location=no,directories=no,status=no,menubar=no,scrollbars=yes,resizable=yes,width=600,height=400");
   cm.document.write("<html><head><title>" + c + "</title></head><body><h2>Comments for candidate " + c + "</h2>" + s + "</body></html>");
   cm.document.close(); 
}
END
return $JSCRIPT;
}

# print a table of statistics listing total number of apps and break down by status
sub print_stats {
	print h3("Statistics");
	my @apps =	get_dirs();
	my %stats;	 
	$stats{total} = scalar @apps;	 
	foreach my $id (@apps) {
		tie(my %cand, 'DB_File', "$id/$id.DB")	||	die "Cannot	open candidate	$id DB_File: $!";
		my $status = $cand{'new'};
		untie %cand;
		$status =~ /\s*(\S*)\s*/;
		$stats{lc($1)}++;
	}
	my @sorted = sort {$stats{$b} <=> $stats{$a}} keys %stats;
	print "<table cellspacing=5 bgcolor=\"aqua\">\n"; 
	print Tr(th("Status"), th("Applications"));
	foreach my $s (@sorted) {
		print Tr({bgcolor => "yellow"}, td($s), td("$stats{$s}")), "\n";
	}
	print "</table>\n"; 
}

# directories, named 0,1,...
sub get_dirs {
	my @dirs;
	opendir(DIR, ".");
	while (defined($_ = readdir(DIR))) {
		if ((-d $_) && (/^(\d+)$/)) {
			push @dirs, $1;
		}
	}
	closedir(DIR);
	return @dirs;
}

# check password against stored encrypted one (make sure file 'passwd' is there!)
# NB: if $passwdflag==0, then the check is passed automatically without any password!
# NB: hope salt is OK ...
sub check {
	return 1 unless $passwdflag; ### NO AUTHENTICATION!!!!
	my $clear = shift;
	return 0 unless defined($clear);
	open(PW, "passwd") || die "passwd file not found: $!";
	my $encoded = <PW>;
	close PW;
	chomp($encoded);
	if (crypt($clear, $encoded) eq $encoded) {
		return 1;
	} else {
		return 0;
	}
}

# create a table entry for reference or extra letter
sub reftable {
	 my ($id, $refno, $tabref, $candref) = @_;
	 
	 my $lnamekey = 'ref'."$refno".'lname';
	 
	 return unless ($candref->{$lnamekey});
	 
	 if ($candref->{$lnamekey} eq 'EXTRA LETTER') {
		 $tabref->{"Reference $refno"} = "Extra letter ";		 
		 $tabref->{"Reference $refno"} .= a({href=>"$URLpath/$id/$id-letter-$refno.txt"}, "[view TXT]") 
			if (-e "$path/$id/$id-letter-$refno.txt");
		 $tabref->{"Reference $refno"} .= a({href=>"$URLpath/$id/$id-letter-$refno.pdf"}, "[view PDF]") 
			if (-e "$path/$id/$id-letter-$refno.pdf");
		 return;		  
	 }
	 
	 my $emailkey = 'ref'."$refno".'email';
	 my $affilkey = 'ref'."$refno".'affil';
	 my $titlekey = 'ref'."$refno".'title';
	 my $fnamekey = 'ref'."$refno".'fname';
	 my $phonekey = 'ref'."$refno".'phone';
	 my $faxkey = 'ref'."$refno".'fax';
	 my $notekey = 'ref'."$refno".'note';
	 
	 $tabref->{"Reference $refno"} = a({href => "mailto:" . $candref->{$emailkey}}, 
		$candref->{$titlekey} . " " . $candref->{$fnamekey} . " " . $candref->{$lnamekey}) . " from " . $candref->{$affilkey} . ' ';
	 $tabref->{"Reference $refno"} .= " Phone: " . $candref->{$phonekey} . " " if ($candref->{$phonekey}); 
	 $tabref->{"Reference $refno"} .= " Fax: " . $candref->{$faxkey} . " " if ($candref->{$faxkey}); 
	 $tabref->{"Reference $refno"} .= a({href=>"$URLpath/$id/$id-letter-$refno.txt"}, "[view TXT]") 
		if (-e "$path/$id/$id-letter-$refno.txt");
	 $tabref->{"Reference $refno"} .= a({href=>"$URLpath/$id/$id-letter-$refno.pdf"}, "[view PDF]") 
		if (-e "$path/$id/$id-letter-$refno.pdf");
	 $tabref->{"Reference $refno"} .= "<br>Note: " . $candref->{$notekey} if ($candref->{$notekey});
}

# transform my score so that '-' and '?' can be sorted numerically
sub rankedscore {
		return -2 if $_[0] eq '?';
		return -1 if $_[0] eq '-';
		return $_[0];
}

# MAIN
if (!param() && !check(cookie('cypher'))) { # stage 0, enter password
	 print header, start_html(-title=>"$dept Recruit"), 
		 h1("Review applications for faculty positions in $dept");
    print start_form();
	 print p("Password:", password_field('secret','',50), submit("Submit!"));
	 print hidden('stage',1);
    print end_form();
	 
} elsif ((!param() && check(cookie('cypher'))) || (param('stage') == 1 && check(param('secret')))) { # stage 1

	 # set the cookie for authenticated user
	 my $passwd;
	 if (!$passwdflag) {
		$passwd = ''; # does not matter what as long as defined
	 } elsif (defined(param('secret'))) { # user just entered password
		$passwd = param('secret');
		print "Set-Cookie: cypher=$passwd\n"; # set the cookie
	 } else { # password from cookie
		$passwd = cookie('cypher');
	 }
	
	 # start page
	 print header, start_html(-title=>"$dept Recruit"), 
		 h2("Review applications for faculty positions in $dept");
     print start_form();
	
	 # save stage and passwd in hidden fields
	 print hidden('secret', $passwd);
	 param('stage',2);
 	 print hidden('stage',2);
 	 
	 # Get reviewer ID and sort preference
 	 my %comm;
	 tie(%comm, 'DB_File', "committee.DB") || die "Cannot open committee DB_File: $!";
	 my %lab = %comm;
	 untie %comm;
	 foreach my $lk (keys %lab) {
		 my @profile = split(/$;/, $lab{$lk});
		 $lab{$lk} = "$lk. $profile[0]";
	 }
	 $lab{''} = 'SELECT YOUR EMAIL'; # add default value as empty string to catch error
	 my @revs = sort (keys %lab); # sorted by rev_id

	 # identify reviewer
	 print p("You <strong>MUST</strong> identify yourself:", popup_menu('rID', \@revs, '', \%lab));

	 # Get reviewer's filter preferences
	 print p(checkbox(-name=>'useprofile', -value=>1, -label=>"Check to review only the areas in reviewer profile"));
	 print p("Or, review the following areas: ", br, checkbox_group('areafilter', $arealist, $arealist, 'true'));
	 print p(checkbox(-name=>'showreject', -value=>1, -label=>"Check to review rejected applications as well"));

	 # sorting preference
	 print p("Sort by:", popup_menu('sort', ['Mean Score','My Score','Submission Date','Applicant Name','Status'], 'Mean Score'));

	 # submit
	 print p(submit("List Applicants!")); # print reset("Clear Form");
    print end_form();
	
	# print some stats
	print_stats();

   } elsif (param('stage') == 2 && length(param('rID')) && check(param('secret'))) { # stage 2

	 # place JavaScript in header for comments pop up window
	 print header, start_html(-title=>"$dept Recruit", -script=>js_showcomment());	
	   
 	 #	choose list	of	applicants to evaluate from table with status info
	 my @apps =	get_dirs();
	 my (%score, %labels, %link, %newflag, %letter, %kw, %kwa, %notes, %nevals, %myscore);
	 my %evals;
	 tie(%evals, 'DB_File', "eval.DB") || die "Cannot open eval DB_File: $!";
	 foreach	my	$id (@apps)	{
		 
		# need candidate DB
		tie(my %cand, 'DB_File', "$id/$id.DB")	||	die "Cannot	open candidate	$id DB_File: $!";
	
		# filter out candidates in unwanted areas or if they are rejected; this is slow!!
		my $filterOK = 0;
		my $matched_areas = 0;	
		my $cand_areas = $cand{'areas'};
		my @filter;
		if (param('useprofile')) {
			# Get reviewer preference
			tie(my %comm, 'DB_File', "committee.DB") || die "Cannot open committee DB_File: $!";
			my @profile = split(/$;/, $comm{param('rID')});
			untie %comm;
			@filter = splice(@profile, 2); # throw away email and notification bit; keep areas
		} else {
			@filter = param('areafilter');
		}
		foreach my $thisarea (@filter) {
			 if ($cand_areas =~ /$thisarea/i) {
				  $filterOK = 1;
				  $matched_areas++;
			 }
		 }
		 $filterOK = 0 if ((!param('showreject')) && ($cand{'new'} =~ /Reject/));		 
		 
		# skip this candidate if there is no area match with reviewer or rejected
		unless ($filterOK) {
			 untie %cand;
			 next; 
		 }
		 
		# check completeness	(glob won't work for user nobody...)
		opendir(DIR, "$path/$id");
		my	@letters	= grep { /^$id-letter-\d\.(txt|pdf)$/ } readdir(DIR);
		$letter{$id} = scalar(@letters);
		closedir(DIR);
		
		# get names, new flag and keywords
		$labels{$id} =  "$cand{'lname'}, $cand{'fname'}";
		$link{$id} = a({href=>"$cand{'url'}"}, $labels{$id}) if length($cand{'url'});
		$newflag{$id} = $cand{'new'};
		$cand_areas =~ s/(['"])/\\$1/g;
		$kw{$id} = $cand_areas;
		$kwa{$id} = $matched_areas;
		
		# done with candidate DB
		untie	%cand;
		
		# get average scores and collated comments
		my $collated = '';
		my ($aver, $N) = (0, 0);
		my $scorebyme = '?';
		while (my($k,$v) = each %evals) {
			$k =~ /(\d*)$;(\d*)/;
			my $thisreviewer = $1;
			if ($2 eq "$id") {
				$collated .= "<p>Reviewer $thisreviewer: ";
				$v =~ /(\d?)$;(.*)/s;
				if (length $1) {
					$aver += $1;
					$N++;
					$scorebyme = $1 if $thisreviewer == param('rID');
				} else {
					$scorebyme = '-' if $thisreviewer == param('rID');
				}
				$collated .= $2 . '\n\n';
			}
		}
		$aver /= $N unless ($N == 0);
		$nevals{$id} = $N;
		$score{$id} = sprintf "%.2f", $aver;
		$collated =~ s/(['"])/\\$1/g;
		$notes{$id} = $collated;
		$myscore{$id} = $scorebyme;
	 }
	 untie %evals;
	 
	# Use all collected into to make a sorted table with checkboxes
	my @sorted;
	@apps = keys(%labels);
	if (param('sort') eq 'Mean Score') {
	 	@sorted = sort {$score{$b} <=> $score{$a}} @apps;
	} elsif (param('sort') eq 'My Score') {
		@sorted = sort {rankedscore($myscore{$b}) <=> rankedscore($myscore{$a})} @apps;
	} elsif (param('sort') eq 'Submission Date') {
	 	@sorted = sort {$b <=> $a} @apps;
	} elsif (param('sort') eq 'Applicant Name') {
	 	@sorted = sort {ucfirst($labels{$a}) cmp ucfirst($labels{$b})} @apps;
	} elsif (param('sort') eq 'Status') {
		@sorted = sort {lc($newflag{$a}) cmp lc($newflag{$b})} @apps;
	} else {
	 	die "Unrecognized sort option\n";
	}
	
	# now that we have the candidates, we can print how many
	my $n_matched = @apps;	
	print h2("$n_matched applications matched in $dept");
    print start_form();
   
	# save stage, passwd and revID in hidden fields
	print hidden('secret', param('secret'));
	print hidden('rID', param('rID'));
	param('stage',3);
	print hidden('stage',3);
	
	# big table
	print "<table border=0 width=100%>\n";
	my $rowcnt = 0;
	foreach my $id (@sorted) {
		print Tr(th(submit("View checked")), th("Applicant name"), th("Ratings (Avg|My|N)"), th("Letters"), 
			th("Matched areas"), th("Status")) if (($rowcnt++ % 10) == 0);
	 	print Tr({-align=>"CENTER", -valign=>"TOP"}, td($id, checkbox(-name=>"check_$id", -value=>1, -label=>'')), 
			td((exists $link{$id}) ? "$link{$id}" : "$labels{$id}"), 
			td(a({href=>"javascript:showcomment('$id', '$notes{$id}')"}, "$score{$id} | $myscore{$id} | $nevals{$id}")), 
			td("$letter{$id}"), 
			td(a({href=>"javascript:onClick=alert('$kw{$id}')"}, $kwa{$id})), 
			td("$newflag{$id}")), 
		"\n";
 	}
	print "</table>\n";
	
	# done with table and form
	#print p({-align=>"CENTER"}), reset("Clear Form"));
    print end_form();
	print p("Rating Legend: ", br, "Avg = Mean score by reviewers who entered a score. ",
		br, "My = My score, '-' if did not enter a score, '?' if did not review. ",
		br, "N = Number of reviews with numerical score.");
	 
} elsif (param('stage') == 3 && check(param('secret'))) { # stage 3

	print header, start_html(-title=>"$dept Recruit"), 
		 h1("Review applications for faculty positions in $dept");
    print start_form();
	
	 # save stage, revID and passwd in hidden fields
	 print hidden('secret', param('secret'));
	 print hidden('rID', param('rID'));
	 param('stage',4);
 	 print hidden('stage',4);
	 
    # get IDs for whom to show details and allow evals
	 my @ids;
	 my @apps =	get_dirs();
	 foreach my $id (@apps) {
	 	push(@ids, $id) if (param("check_$id"));
 	 }

	 # PARAMS ORDERED FOR CONVENIENCE 
	 my @parameters = ("First Name", "Last Name", "Status", "Application Date", "Homepage", "Email", "Fax", "Phone", 
		 "Address", "Affiliation", "PhD Degree", "PhD Institution", "PhD Date", "Areas");
	 
	 # for each selected candidate, show database info (linking letters if there) and pdf files
	 my %labels;
	 foreach my $id (sort {$a <=> $b} @ids) {
	 	print hr, h2("Information for Applicant $id");
	 	
	 	# read info from DB
		my (%cand, %table);
		tie(%cand, 'DB_File', "$id/$id.DB") || die "Cannot open candidate $id DB_File: $!";
		$labels{$id} = "$cand{'lname'}, $cand{'fname'}";
		
		$table{"First Name"} = $cand{'fname'};
		$table{"Last Name"} = $cand{'lname'};
		$table{"Status"} = $cand{'new'};
		$table{"Application Date"} = $cand{'date'};
		$table{"Homepage"} = a({href=>"$cand{'url'}"}, $cand{'url'});
		$table{"Email"} = a({href=>"mailto:$cand{'email'}"}, $cand{'email'}); 
		$table{"Fax"} = $cand{'fax'};
		$table{"Phone"} = $cand{'phone'};
		$table{"Address"} = $cand{'address'};
		$table{"Affiliation"} = $cand{'affil'};
		$table{"PhD Degree"} = $cand{'dept'};
		$table{"PhD Institution"} = $cand{'school'};
		$table{"PhD Date"} = "$cand{'phdmonth'}/$cand{'phdyear'}";
		$table{"Areas"} = $cand{'areas'};
		
		# create table entries for references and extra letters
		opendir(DIR, "$path/$id");
		my @rnumsa = (1..6); # only the references
		my %rnumsh; 
		@rnumsh{@rnumsa} = ();
		# add extras and label them now (this really should be done at inserion time in admin...)
		while (defined($_ = readdir(DIR))) { 
		   if (/^$id-letter-(\d+)\./) {
			   $cand{'ref'.$1.'lname'} = 'EXTRA LETTER' unless $cand{'ref'.$1.'lname'};
			   $rnumsh{$1} = undef; 
		   }
		}
		closedir(DIR);
		
		# now both refs and extra
		@rnumsa = sort keys %rnumsh; 
		foreach my $rnum (@rnumsa) {
			 reftable($id, $rnum, \%table, \%cand);
		}
		
		# done with this candidate
		untie %cand;
		
		# format and output in nice table
		print "<TABLE>\n";
		foreach my $key (@parameters) {
			print(Tr(th($key), td($table{$key}))) if exists $table{$key};
		}
		foreach my $rnum (@rnumsa) {
			my $key = "Reference $rnum";
			print(Tr(th($key), td($table{$key}))) if exists $table{$key};
		}
		
	 	# add links to 2 pdf files
		my $s;
		if (-e "$path/$id/$id-cvfile.pdf") {
			$s = a({href=>"$URLpath/$id/$id-cvfile.pdf"}, "view");
		} else {
			$s = "not available";
		}
		print Tr(th("CV"), td($s));
		if (-e "$path/$id/$id-stmtfile.pdf") {
			$s = a({href=>"$URLpath/$id/$id-stmtfile.pdf"}, "view");
		} else {
			$s = "not available";
		}
		print Tr(th("Statement"), td($s));
		
		# allow evaluation of this applicant by a registered reviewer
		my %rating_labels = (
			'0' => '0: Poor',
			'1' => '1: Fair',
			'2' => '2: Good',
			'3' => '3: Excellent',
			'4' => '4: Outstanding',
			'UNRATED' => 'Cannot Rate'
		);
		print Tr(th("Grade"), 
			td(popup_menu("grade_$id", [0,1,2,3,4,'UNRATED'], 'UNRATED', \%rating_labels), 
			checkbox(-name=>"check_$id", -label=>'Check to evaluate this applicant')));
		print Tr(th("Comments"), td(textarea("comment_$id",'',4,50)));
		 
		# done with this applicant
		print "</TABLE>\n";
	 }
	 
	 #print p(submit("Submit Evaluations!"), reset("Clear Form"));
	 print p(submit("Submit Evaluations!"));
    print end_form();
    
} elsif (param('stage') == 4 && check(param('secret'))) { # stage 4

	# enter record into intersection table, format: 'rID';'aID'=>'grade';'comment'
	my %evals;
	tie(%evals, 'DB_File', "eval.DB") || die "Cannot open eval DB_File: $!";
	my @ids;
	my @apps =	get_dirs();
	foreach my $id (@apps) {
		if (param("check_$id")) {
			my $this_grade = param("grade_$id");
			#die "You did not enter a valid grade for applicant $id\n" if $this_grade eq 'UNRATED';
			$this_grade = '' if $this_grade eq 'UNRATED';
			$evals{param('rID') . $; . $id} = $this_grade . $; . param("comment_$id");
		}
 	}
	untie %evals;

	print header, start_html(-title=>"$dept Recruit"), 
		 h1("Review applications for faculty positions in $dept");
	print p("Thank you for your evaluation(s).");
	print "You may ", a({href=>"review.cgi"}, "evaluate additional applications"), 
		" or return to the ", a({href=>"../index.cgi"}, "Recruit Home"), ".\n";

} else { # FAIL STAGE

	print header, start_html(-title=>"$dept Recruit"), 
		 h1("Review applications for faculty positions in $dept");
 	print h2("A problem occurred. Maybe you did not identify yourself, or you entered the wrong password.");
 	
}
print hr, a({href=>"http://www.informatics.indiana.edu/fil/Recruit/", target=>"_new"}, 'Recruit'), " &copy; 2000-2004 ", a({href=>"http://www.informatics.indiana.edu/fil/", target=>"_new"}, 'Filippo Menczer'); 
print end_html;

